@@ -7,12 +7,12 @@ from dploi_fabric.utils import config
 import posixpath
 
 @task
-def update_config_file():
-    template = 'templates/supervisor/supervisor.conf'
-    group_template = 'templates/supervisor/supervisor-group.conf'
+def update_config_file(dryrun=False):
     output = ''
     groups = {}
     for site, site_config in config.sites.items():
+        template_path = site_config['supervisor']['template']
+        group_template_path = site_config['supervisor']['group_template']
         group_name = get_group_name(site, site_config)
         groups[group_name] = []
         for process_name, process_dict in site_config.processes.items():
@@ -34,11 +34,16 @@ def update_config_file():
                 'priority': process_dict.get('priority', 200),
                 'autostart': 'True' if getattr(env, 'autostart', True) else 'False',
             })
-            output += render_template(template, context_dict)
+            output += render_template(template_path, context_dict)
             groups[group_name].append(process_name)
-    output += render_template(group_template, {'groups': groups})
-    put(StringIO.StringIO(output), '%(path)s/../config/supervisor.conf' % config.sites["main"].deployment)
-    update()
+    output += render_template(group_template_path, {'groups': groups})
+    path = posixpath.abspath(posixpath.join(config.sites["main"].deployment['path'], '..', 'config', 'supervisor.conf'))
+    if dryrun:
+        print path + ':'
+        print output
+    else:
+        put(StringIO.StringIO(output), path)
+        update()
 
 @task
 def stop():
