@@ -1,3 +1,4 @@
+from dploi_fabric.toolbox.template import app_package_path, render_template
 import os
 import posixpath
 
@@ -31,12 +32,17 @@ class Configuration(object):
         'checkout': {
             'tool': 'buildout',
         },
+        'gunicorn': {
+            'workers': 2,
+        },
         'celery': {
             'enabled': False,
             'concurrency': 1,
             'maxtasksperchild': 500,
             'loglevel': 'WARNING',
             'celerycam': False,
+            'version': None,
+            'app': 'project',
         },
         'static': {
 
@@ -44,6 +50,7 @@ class Configuration(object):
         'redis': {
             'enabled': False,
             'appendonly': 'no',
+            'template': app_package_path('templates/redis/redis.conf'),
         },
         'processes': {
 
@@ -55,7 +62,21 @@ class Configuration(object):
 
         },
         'nginx': {
-            'client_max_body_size': '10m'
+            'client_max_body_size': '10m',
+            'template': app_package_path('templates/nginx/nginx.conf'),
+        },
+        'supervisor': {
+            'template': app_package_path('templates/supervisor/supervisor.conf'),
+            'group_template': app_package_path('templates/supervisor/supervisor-group.conf'),
+            'gunicorn_command_template': app_package_path('templates/supervisor/gunicorn_command'),
+            'celeryd_command_template': app_package_path('templates/supervisor/celeryd_command'),
+            'celerycam_command_template': app_package_path('templates/supervisor/celerycam_command'),
+        },
+        'newrelic': {
+            'enabled': False,
+            'config_file': 'newrelic.ini',
+            'environment_name': '',
+            'license': '',
         }
     }
     def load_sites(self, config_file_content=None, env_dict=None):
@@ -64,13 +85,16 @@ class Configuration(object):
         and their individual settings.
         """
         if not config_file_content:
-            config_file = os.path.join(env.path, "config.ini")
-            if exists(config_file):
-                output = StringIO.StringIO()
-                get(u"%s" % config_file, output)
-                output.seek(0)
+            if env.get("use_local_config_ini", False):
+                output = open("config.ini")
             else:
-                raise Exception("Missing config.ini, tried path %s" % config_file)
+                config_file = os.path.join(env.path, "config.ini")
+                if exists(config_file):
+                    output = StringIO.StringIO()
+                    get(u"%s" % config_file, output)
+                    output.seek(0)
+                else:
+                    raise Exception("Missing config.ini, tried path %s" % config_file)
         else:
             output = StringIO.StringIO(config_file_content)
 
@@ -128,7 +152,11 @@ class Configuration(object):
                 attr_dict["django"]["cmd"] = " ".join(new_django_cmd)
                 if attr_dict["django"]["append_settings"]:
                     attr_dict["django"]["args"].append(" --settings=%s" % ('_gen.settings', ))
-
+            if attr_dict["newrelic"]["enabled"]:
+                attr_dict["django"]["cmd"] = posixpath.join(
+                    attr_dict.get("deployment").get("path"),
+                    "bin/newrelic-admin"
+                ) +  " run-program " + attr_dict["django"]["cmd"]
             attr_dict.update({'processes': self.processes(site, env_dict)})
             attr_dict['environment'] = self.environment(site, env_dict)
             attr_dict['environment'].setdefault('DEPLOYMENT_SITE', site)
@@ -168,13 +196,21 @@ class Configuration(object):
         """
         process_dict = {}
         site_dict = self.sites[site]
-        django_args = " ".join(site_dict.get("django").get("args", []))
-        gunicorn_socket = posixpath.normpath(posixpath.join(env_dict.get("path"), "..", "tmp", "%s_%s_gunicorn.sock" % (env_dict.get("user"), site))) # Asserts pony project layout
-
+        common_cmd_context = {
+            "django_cmd": site_dict.django['cmd'],
+            "django_args": " ".join(site_dict.get("django").get("args", [])),
+        }
+        gunicorn_cmd_context = {
+            "socket": posixpath.normpath(posixpath.join(env_dict.get("path"), "..", "tmp", "%s_%s_gunicorn.sock" % (env_dict.get("user"), site))), # Asserts pony project layout
+            "workers": site_dict.gunicorn['workers'],
+        }
+        gunicorn_cmd_context.update(common_cmd_context)
+        gunicorn_command_template_path = self.sites[site]['supervisor']['gunicorn_command_template']
+        gunicorn_command = render_template(gunicorn_command_template_path, gunicorn_cmd_context)
         process_dict["%s_%s_gunicorn" % (env_dict.get("user"), site)] = {
-                    'command': "%s run_gunicorn %s -w 2 -b unix:%s" % (site_dict.django['cmd'], django_args, gunicorn_socket),
+                    'command': gunicorn_command,
                     'port': None,
-                    'socket': gunicorn_socket,
+                    'socket': gunicorn_cmd_context['socket'],
                     'type': 'gunicorn',
                     'priority': 100,
                 }
@@ -189,26 +225,41 @@ class Configuration(object):
                     'priority': 60,
                 }
         if site_dict.get("celery").get("enabled"):
+            cmd = env_dict.get("path") if not site_dict.get("newrelic").get("enabled") else '%sbin/newrelic-admin run-program %s' % (env_dict.get("path"), env_dict.get("path"))
+            cmd += 'bin/celery'
+            celeryd_command_context = {
+                'concurrency': site_dict.get("celery").get("concurrency"),
+                'maxtasksperchild': site_dict.get("celery").get("maxtasksperchild"),
+                'loglevel': site_dict.get("celery").get("loglevel"),
+                'path': env_dict.get("path"),
+                'version': site_dict.get("celery").get("version"),
+                'celery_app': site_dict.get("celery").get("app"),
+                'has_cam': site_dict.get("celery").get("celerycam"),
+                'cmd': cmd,
+            }
+            celeryd_command_context.update(common_cmd_context)
+            celeryd_command_template_path = self.sites[site]['supervisor']['celeryd_command_template']
+            celeryd_command = render_template(celeryd_command_template_path, celeryd_command_context)
             process_dict["%s_%s_celeryd" % (env_dict.get("user"), site)] = {
-                    'command': "%s celeryd %s -E -B -c %s --maxtasksperchild %s --loglevel=%s" % (
-                        site_dict.django['cmd'],
-                        django_args,
-                        site_dict.get("celery").get("concurrency"),
-                        site_dict.get("celery").get("maxtasksperchild"),
-                        site_dict.get("celery").get("loglevel"),
-                    ),
+                    'command': celeryd_command,
                     'port': None,
                     'socket': None,
                     'type': 'celeryd',
                     'priority': 40,
                 }
             if site_dict.get("celery").get("celerycam"):
+                celerycam_command_context = {
+                    'loglevel': site_dict.get("celery").get("loglevel"),
+                    'path': env_dict.get("path"),
+                    'version': site_dict.get("celery").get("version"),
+                    'celery_app': site_dict.get("celery").get("app"),
+                    'cmd': cmd,
+                }
+                celerycam_command_context.update(common_cmd_context)
+                celerycam_command_template_path = self.sites[site]['supervisor']['celerycam_command_template']
+                celerycam_command = render_template(celerycam_command_template_path, celerycam_command_context)
                 process_dict["%s_%s_celerycam" % (env_dict.get("user"), site)] = {
-                    'command': "%s celerycam %s --loglevel=%s" % (
-                        site_dict.django['cmd'],
-                        django_args,
-                        site_dict.get("celery").get("loglevel"),
-                    ),
+                    'command': celerycam_command,
                     'port': None,
                     'socket': None,
                     'type': 'celerycam',
@@ -304,16 +355,86 @@ class Configuration(object):
         deployment_dict.update({'domains': domains})
 
         ###############
-        # Celery dict #
+        # Environment #
         ###############
 
+        environment_dict = self.sites[site].get("environment")
+        for key, value in env_dict.get("environment", {}).items():
+            environment_dict[key] = value
+
+        #################
+        # Gunicorn dict #
+        #################
+        gunicorn_dict = self.sites[site].get("gunicorn")
+        gunicorn_dict["workers"] = env_dict.get("gunicorn", {}).get("workers", gunicorn_dict.get("workers"))
+
+        ###############
+        # Celery dict #
+        ###############
         celery_dict = self.sites[site].get("celery")
 
         celery_dict["concurrency"] = env_dict.get("celery", {}).get("concurrency", celery_dict.get("concurrency"))
         celery_dict["maxtasksperchild"] = env_dict.get("celery", {}).get("maxtasksperchild", celery_dict.get("maxtasksperchild"))
 
+        ##############
+        # nginx dict #
+        ##############
 
-        return {'deployment': deployment_dict, 'celery': celery_dict}
+        nginx_dict = self.sites[site].get("nginx")
+        nginx_dict["location_settings"] = {
+            "client_max_body_size": env_dict.get("nginx", {}).get("client_max_body_size", nginx_dict.get("client_max_body_size")),
+        }
+        nginx_dict["template"] = env_dict.get("nginx", {}).get("template", nginx_dict.get("template"))
+
+        ##############
+        # redis dict #
+        ##############
+
+        redis_dict = self.sites[site].get("redis")
+        redis_dict["template"] = env_dict.get("redis", {}).get("template", redis_dict.get("template"))
+
+        ###################
+        # supervisor dict #
+        ###################
+
+        supervisor_dict = self.sites[site].get("supervisor")
+        supervisor_dict["template"] = env_dict.get("supervisor", {}).get("template", supervisor_dict.get("template"))
+        supervisor_dict["group_template"] = env_dict.get("supervisor", {}).get("group_template", supervisor_dict.get("group_template"))
+        supervisor_dict["gunicorn_command_template"] = env_dict.get("supervisor", {}).get("gunicorn_command_template", supervisor_dict.get("gunicorn_command_template"))
+        supervisor_dict["celeryd_command_template"] = env_dict.get("supervisor", {}).get("celeryd_command_template", supervisor_dict.get("celeryd_command_template"))
+        supervisor_dict["celerycam_command_template"] = env_dict.get("supervisor", {}).get("celerycam_command_template", supervisor_dict.get("celerycam_command_template"))
+
+        #################
+        # newrelic dict #
+        #################
+
+        newrelic_dict = self.sites[site].get("newrelic")
+        newrelic_dict["enabled"] = env_dict.get("newrelic", {}).get("enabled", newrelic_dict.get("enabled"))
+        newrelic_dict["config_file"] = env_dict.get("newrelic", {}).get("config_file", newrelic_dict.get("config_file"))
+        if not newrelic_dict["config_file"].startswith('/'):
+            newrelic_dict["config_file"] = posixpath.abspath(posixpath.join(
+                    deployment_dict["path"],
+                    newrelic_dict["config_file"],
+                ))
+        self.sites[site]["environment"]["NEW_RELIC_CONFIG_FILE"] = newrelic_dict["config_file"]
+        newrelic_dict["environment_name"] = env_dict.get("newrelic", {}).get("environment_name", newrelic_dict.get("environment_name"))
+        if newrelic_dict["environment_name"]:
+            self.sites[site]["environment"]["NEW_RELIC_ENVIRONMENT"] = newrelic_dict["environment_name"]
+
+        newrelic_dict["license_key"] = env_dict.get("newrelic", {}).get("license_key", newrelic_dict.get("license_key"))
+        if newrelic_dict["license_key"]:
+            self.sites[site]["environment"]["NEW_RELIC_LICENSE_KEY"] = newrelic_dict["license_key"]
+
+        return {
+            'deployment': deployment_dict,
+            'environment': environment_dict,
+            'gunicorn': gunicorn_dict,
+            'celery': celery_dict,
+            'nginx': nginx_dict,
+            'redis': redis_dict,
+            'supervisor': supervisor_dict,
+            'newrelic': newrelic_dict,
+        }
 
     def django_manage(self, command, site="main"):
         """
@@ -356,10 +477,10 @@ def ps():
     run('ps -f -u %(user)s | grep -v "ps -f" | grep -v sshd' % env)
 
 @task
-def download_media(to_dir="./tmp/media/", from_dir="../upload/"):
+def download_media(to_dir="./tmp/media/", from_dir="../upload/media/"):
     """
     Downloads media from a remote folder, default ../uploads/ -> ./tmp/media/
-    
+
     * Example: upload_media:from_dir="py_src/project/media/"
     """
     print "Downloading media from", env.host_string
@@ -367,7 +488,7 @@ def download_media(to_dir="./tmp/media/", from_dir="../upload/"):
     local('rsync -avz --no-links --progress --exclude=".svn" -e "ssh" %(user)s@%(host_string)s:"%(path)s/%(from_dir)s"' % env + " " +to_dir)
 
 @task
-def upload_media(from_dir="./tmp/media/", to_dir="../upload/"):
+def upload_media(from_dir="./tmp/media/", to_dir="../upload/media/"):
     """
     Uploads media from a local folder, default ./tmp/media -> ../uploads/
     
@@ -376,3 +497,8 @@ def upload_media(from_dir="./tmp/media/", to_dir="../upload/"):
     print "Uploading media to", env.host_string
     env.to_dir = to_dir
     local('rsync -avz --no-links --progress --exclude=".svn" '+ from_dir +' -e "ssh" %(user)s@%(host_string)s:"%(path)s/%(to_dir)s"' % env)
+
+
+@task
+def use_local_config_ini():
+    env.use_local_config_ini = True
\ No newline at end of file
