@@ -16,3 +16,7 @@ def create():
     """
     do_run('cd %(path)s; virtualenv . --system-site-packages --setuptools' % config.sites["main"].deployment)
     update()
+    # this is ugly. I know. But it seems that on first run, pip does not
+    # install the correct version of packages that are pulled directly from
+    # git. Only the second time around it uses the correct one.
+    update()
